<?php

/**
 * Salesfire Tracking
 *
 * @package     Salesfire_Tracking
 * @version     1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class Salesfire_Tracking
{
    private $script = '';

    private $events = [];

    private $data_layer_injected = false;

    public function init() {
        if (wp_doing_cron() || is_admin() || ! get_option('salesfire_tracking')) {
            return;
        }

        $this->base_tag();

        if (class_exists( 'WooCommerce' )) {
            if ( ! wp_doing_ajax() ) {
                add_action( 'wp', array( $this, 'product_event' ) );
            }

            add_action( 'woocommerce_before_thankyou', array( $this, 'order_event' ), 10, 1 );
        }

        // Print to head.
        add_action( 'wp_head', array( $this, 'print_script' ) );
    }

    private function base_tag() {
        if ($uuid = get_option('salesfire_uuid')) {
            $this->script .= sprintf(
                '<script async src="https://cdn.salesfire.co.uk/code/%s.js"></script>',
                esc_attr( $uuid )
            );
        }
    }

    /**
     * Enqueues the product page event code for printing.
     *
     * @return void
     */
    public function product_event()
    {
        if ( is_product() ) {
            $product = wc_get_product();

            $data = (object)[
                "ecommerce" => (object)[
                    "view" => (object)[
                        "sku"           => $product->get_id(),
                        "parent_sku"    => $product->get_parent_id(),
                        "name"          => $product->get_title(),
                        "variant"       => $product->get_description(),
                        "price"         => $product->get_price(),
                        "currency"      => "GBP",
                    ]
                ]
            ];

            $this->add_event( $data );
        }
    }

    /**
     * Enqueues the order event code for printing.
     *
     * @return void
     */
    public function order_event($order_id)
    {
        $order = wc_get_order( $order_id );

        if ( ! $order ) {
            return;
        }

        $data = (object)[
            "ecommerce" => (object)[
                "purchase" => (object)[
                    "id"        => $order_id,
                    "revenue"   => $order->get_subtotal(),
                    "shipping"  => $order->get_shipping_total(),
                    "tax"       => $order->get_tax_totals(),
                    "currency"  => $order->get_currency(),
                    "products"  => [],
                ],
            ],
        ];

        foreach ( $order->get_items() as $order_item ) {
            if ( ! method_exists( $order_item, 'get_product' ) ) {
                continue;
            }

            $product = $order_item->get_product();

            $data->ecommerce->purchase->products[] = (object)[
                "sku"           => $order_item->get_id(),
                "parent_sku"    => $product->get_parent_id(),
                "name"          => $product->get_title(),
                "variant"       => $product->get_description(),
                "price"         => $product->get_price(),
                "currency"      => $order->get_currency(),
            ];
        }

        $this->add_event( $data );
    }

    /**
     * Enqueues or prints the given event, depending on if
     * we have already run wp_head or not.
     *
     * @param string $event The event's type.
     * @param array  $data  The data to be passed to the JS function.
     *
     * @return void
     */
    private function add_event( $event )
    {
        $this->inject_layer();

        if (did_action('wp_head')) {
            echo '<script>window.sfDataLayer.push(' . json_encode($event) . ');</script>';
        } else {
            $this->events[] = $event;
        }
    }

    public function inject_layer()
    {
        if (! $this->data_layer_injected) {
            echo '<script>window.sfDataLayer = window.sfDataLayer || [];</script>';

            $this->data_layer_injected = true;
        }
    }

    public function print_script()
    {
        if (! empty($this->script)) {
            echo $this->script;

            if (! empty($this->events)) {
                $this->inject_layer();

                echo '<script>';

                foreach ($this->events as $event) {
                    echo 'window.sfDataLayer.push(' . json_encode($event) . ');';
                }

                echo '</script>';
            }
        }
    }
}
